"""
 Copyright (c) 2015 Michael Bright and Bamboo HR LLC

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
"""


class Communicator(object):

    def __init__(self, server_uri, flask_app=None):
        self.server_uri = server_uri
        self.flask_app = flask_app

    def get_uri(self, uri_stem):
        return self.server_uri + uri_stem

    def api_uri(self, uri_stem):
        return self.get_uri("/api/{}".format(uri_stem))
